//+------------------------------------------------------------------+
//|                                       PlotActivationFunction.mq5 |
//|                                  Copyright 2023, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2023, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"

#include <Graphics\Graphic.mqh>

#property script_show_inputs
//--- input parameters
input ENUM_ACTIVATION_FUNCTION InpAF=AF_ELU;
input double                   InpStart=-5.0;
input double                   InpStep=0.01;
input double                   InpStop=5.0;
input double                   InpParameter1=0;
input double                   InpParameter2=0;
input double                   InpParameter3=0;

#define GRAPH_WIDTH  750
#define GRAPH_HEIGHT 350

//+------------------------------------------------------------------+
//| Script program start function                                    |
//+------------------------------------------------------------------+
void OnStart()
  {
//--- vector of successively increasing values
   vector vector_a,vector_c,vector_d;
   VectorFill(vector_a,InpStart,InpStep,InpStop);

//--- vector with activation function results
   if(!vector_a.Activation(vector_c,InpAF,InpParameter1,InpParameter2,InpParameter3))
     {
      PrintFormat("error %d in %s",GetLastError(),EnumToString(InpAF));
      return;
     }

//--- vector with derivatives of activation function
   if(!vector_a.Derivative(vector_d,InpAF,InpParameter1,InpParameter2,InpParameter3))
     {
      PrintFormat("error %d in %s",GetLastError(),EnumToString(InpAF));
      return;
     }

//--- switch off chart show
   ChartSetInteger(0,CHART_SHOW,false);

//--- arrays for drawing a graph
   double x[];
   double y1[];
   double y2[];
   ArrayResize(x,uint(vector_a.Size()));
   ArrayResize(y1,uint(vector_a.Size()));
   ArrayResize(y2,uint(vector_a.Size()));
   for(ulong i=0; i<vector_a.Size(); i++)
     {
      x[i]=vector_a[i];
      y1[i]=vector_c[i];
      y2[i]=vector_d[i];
     }

//--- graph title
   string title=EnumToString(InpAF);
   if(InpParameter1!=0)
      title+=", "+string(InpParameter1);
   if(InpParameter2!=0)
      title+=", "+string(InpParameter2);
   if(InpParameter3!=0)
      title+=", "+string(InpParameter3);

   long   chart=0;
   string name="GraphicActivation";

//--- create graph
   CGraphic graphic;
   graphic.Create(chart,name,0,0,0,GRAPH_WIDTH,GRAPH_HEIGHT);
   graphic.BackgroundMain(title);
   graphic.BackgroundMainSize(12);
   
//--- activation function graph
   CCurve *curvef=graphic.CurveAdd(x,y1,CURVE_LINES);
   curvef.Name("af");
   curvef.LinesWidth(2);
   curvef.LinesSmooth(true);
   curvef.LinesSmoothTension(1);
   curvef.LinesSmoothStep(10);

//--- derivatives of activation function
   CCurve *curved=graphic.CurveAdd(x,y2,CURVE_LINES);
   curved.Name("f'");
   curved.LinesWidth(2);
   curved.LinesSmooth(true);
   curved.LinesSmoothTension(1);
   curved.LinesSmoothStep(10);
   graphic.CurvePlotAll();
   graphic.Update();

//--- endless loop to recognize pressed keyboard buttons
   while(!IsStopped())
     {
      //--- press escape button to quit program
      if(TerminalInfoInteger(TERMINAL_KEYSTATE_ESCAPE)!=0)
         break;
      //--- press PdDn to save graph picture
      if(TerminalInfoInteger(TERMINAL_KEYSTATE_PAGEDOWN)!=0)
        {
         string file_names[];
         if(FileSelectDialog("Save Picture",NULL,"All files (*.*)|*.*",FSD_WRITE_FILE,file_names,EnumToString(InpAF)+".png")<1)
            continue;
         ChartScreenShot(0,file_names[0],GRAPH_WIDTH,GRAPH_HEIGHT);
        }
      Sleep(10);
     }

//--- clean up
   graphic.Destroy();
   ObjectDelete(chart,name);
   ChartSetInteger(0,CHART_SHOW,true);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void VectorFill(vector& v,const double start,const double step,const double stop)
  {
   ulong size=ulong((stop-start)/step)+1;
   double init=start;

   v.Resize(size);
   for(ulong i=0; i<size; i++,init+=step)
      v[i]=init;
  }
//+------------------------------------------------------------------+
